#!/bin/sh

 #######################################################################
 # Copyright 2007-2009 Igor Daniloff.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 #######################################################################


#vars 

if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=grep
fi

if [ -f /usr/xpg4/bin/egrep ]; then
    EGREP=/usr/xpg4/bin/egrep
else
    EGREP=egrep
fi

DEBUG="no"
#ROOT_PATH=""
DEFAULT_BIN_PATH=/opt/drweb
DEFAULT_BIN_PATH=`echo "$DEFAULT_BIN_PATH" | $SED 's/^\///'`
SCRIPTS_DIR="$DEFAULT_BIN_PATH/maild/scripts"
DOC_DIR="$DEFAULT_BIN_PATH/doc/maild"
DEFAULT_VAR_PATH="$ROOT_PATH/var/drweb"
if [ "0" = "1" ]; then
   INIT_SCRIPT="$ROOT_PATH/usr/local/etc/rc.d/00.drweb-monitor.sh"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/usr/local/drweb/lib"
else
   INIT_SCRIPT="$ROOT_PATH/etc/init.d/drweb-monitor"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/opt/drweb/lib"
fi

#functions
######################################################################
dwd()
{
    if [ "$DEBUG" = "yes" ]; then
        echo $1
    fi
}
######################################################################
DwFindFullPath()
{
    if [ -x "./$0" ] ; then
        var=`echo $0`
        var=`echo "$var" | $SED 's/^\.\///'`
        SCRIPT_PATH=`pwd`/"$var"
        SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\"//g'`
    else
        SCRIPT_PATH="$0"
    fi
    SCRIPT_NAME=`echo "$SCRIPT_PATH" | $SED 's/^.*\/\([^\/]*\)$/\1/'`
    SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\/[^\/]*$//'`
    dwd "SCRIPT_PATH=$SCRIPT_PATH SCRIPT_NAME=$SCRIPT_NAME"
}

######################################################################
DwInstall()
{
(cd "$1"
    CHOWN=""
    ADD="no"
    for element in *; do
        if [ -d "$element" ]; then
            if [ ! -d "$2/$element" ]
            then
                if [ "$3" = "modif" -o "$3" = "update" ]
                then
                    mkdir -p "$2/$element" || exit 1
                    chown drweb:drweb "$2/$element" || exit 1
                    chmod 0775 "$2/$element" || exit 1
                    ADD="yes"
                    dwd "$element => $2/$element"
                fi
            fi
            DwInstall $element "$2/$element" $3 || exit 1
            if [ "$ADD" = "yes" ]
            then
                echo "$2/$element" >> "$DW_INSTALL_LOG"
            fi
        fi
        suffix=""
        if [ -f "$element"  -a "$element" != "$SCRIPT_NAME"  -a "$element" != "update.sh" ]; then
            ext=`echo "$element" | $SED 's/^.*\.\([^\.]*\)$/\1/'`
            if [ -f "$2/$element" ]; then
                if [ "$element" = "emails" -o "$ext" = "conf" -o "$ext" = "ini" -o "$ext" = "mmc" ]; then
                    if [ "$3" = "modif" ]
                    then
                        echo "File $2/$element exist: save it to $2/$element.drwebsave"
                        mv "$2/$element" "$2/$element.drwebsave" || exit 1
                    elif [ "$3" = "update" ]
                    then
                        echo "File $2/$element exist: save new version to $2/$element.new"
                        suffix=".new"
                    fi
                elif [ "$3" != "update" ]; then
                    echo "File $2/$element already exist: can not continue!"
                    exit 1
                fi
            fi
            if [ "$3" != "fake" ]
            then
                if [ ! -f "$2/$element$suffix" ]; then
                    echo "$2/$element$suffix" >> "$DW_INSTALL_LOG"
                    dwd "$element ($ext) => $2/$element$suffix"
                fi
                if [ -L "$element" ]; then
                    STAT_INFO=`ls -l "$element" | $SED "s/^.*-> \(.*\)$/\1/"`
                    (cd "$2"; if [ -f "$element$suffix" ]; then rm -f "$element$suffix"; fi; ln -s "$STAT_INFO" "$element$suffix"|| exit 1) || exit 1
                else
                    cp -p -R "$element" "$2/$element$suffix" || exit 1
                fi
                CHOWN="$CHOWN $2/$element$suffix"
            fi
        fi
    done
    if [ ! -z "$CHOWN" ]; then
        chown drweb:drweb $CHOWN || exit 1
    fi
)
}
######################################################################
DwStop()
{
    if [ ! -f $INIT_SCRIPT ]; then
        echo "Can not find init script $INIT_SCRIPT"
        exit 2
    fi
    
    $INIT_SCRIPT status
    if [ $? -eq 0 ]; then
        $INIT_SCRIPT stop
        if [ $? -ne 0 ]; then
            echo "Can not stop Dr.Web MailD"
            exit 3
        fi
    fi
}
######################################################################
DwReload()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT reload
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers reloaded"
            else
                echo "" 
                echo "Automatic reload failed."
                echo "Please inspect Dr.Web for Unix mail servers and plugins configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    fi
}
######################################################################
DwRestart()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT restart
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers restarted"
            else
                echo "" 
                echo "Automatic start failed."
                echo "Please inspect Dr.Web for Unix mail servers configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    elif [ -x "`which svcadm`" ] && svcs drweb-monitor >/dev/null 2>&1; then
        # Would not restart if disabled currently.
        svcadm restart drweb-monitor
    else
        echo ""
        echo "Dr.Web for Unix mail servers restart failed."
        echo "Neither SysV nor SVC service was found on your system."
    fi
}

######################################################################
DwBackup()
{
    if [ -f "$1" ] ; then

        if [ -f "$1.drwebsave" ] ; then

            CONF_DRWEBSAVE="yes"
            TIMESTAMP=`date "+%Y%m%d-%k_%M_%S"`
            echo ""
            echo "Info: $1.drwebsave exists. Saving $1 as $1.$TIMESTAMP"
            echo ""
            cp "$1" "$1.$TIMESTAMP"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 3
            fi
        fi
        if [ "$CONF_DRWEBSAVE" != "yes" ] ; then
            cp "$1" "$1.drwebsave"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 2
            else
                echo ""
                echo "Success: backup of $1 in $1.drwebsave."
                echo ""
            fi

        fi
    else
        echo ""
        echo "Error: can't find $1!"
        echo ""
        return 1
    fi
    return 0;
}
######################################################################
DwPostProc()
{
    chmod +s "$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-inject"
}

######################################################################
DwInstallLibs()
{
:
#     for file in libstdc++.so.6 libgcc_s.so.1
#     do
#       if [ ! -f $SYS_LIB_DIR/$file ]; then
#           dwd "Make link from $DW_LIB_DIR/$file to $SYS_LIB_DIR/$file"
#           rm -f $SYS_LIB_DIR/$file >/dev/null 2>&1
#           ln -s $DW_LIB_DIR/$file $SYS_LIB_DIR/$file || exit 1
#       fi.
#     done
}
######################################################################
DwSetPluginName()
{
    OLD=`pwd`
    if [ ! -d "/$PLUGINS_DIR" ];then
        echo "Can not find plugin directory '/$PLUGINS_DIR'"
        exit 1
    fi
    cd "/$PLUGINS_DIR"
    for plugin in *; do
        PLUGIN_NAME=`echo "$plugin" | $SED 's/^lib\(.*\)\.so$/\1/'`
        dwd "PLUGIN_NAME = $PLUGIN_NAME"
        break
    done
    cd "$OLD"
    if [ "x$PLUGIN_NAME" = "x*" ]; then
        echo "Can not find any plugin library in /$PLUGINS_DIR"
        exit 1
    fi
}

DwConfigureMonitor()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/User = drweb/User = root/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                if test -z $POSTINSTALL_FLAG; then
                    DwRestart
                fi
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change User = drweb to User = root manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

DwConfigureMonitorRunAppList()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/RunAppList = MAILD/RunAppList = MAILDESK/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                DwRestart
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change 'RunAppList = MAILD' to 'RunAppList = MAILDESK' manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

############ PRE ##############

#root

id | $EGREP "uid=0\(" 1> /dev/null
if [ $? -ne 0 ]; then
    echo "You must be root to run the Dr.Web for Unix mail servers installer."
    exit 1
fi

#reset umask
umask 0002

cd "/$SCRIPTS_DIR"

DwFindFullPath


#vars
MTA_CONFIGURE="no"
DW_INSTALL_LOG="$DEFAULT_VAR_PATH/maild_SENDMAIL_install.info"

DwPrepareMta()
{
    #fix #14359: for sendmail milter socket
    chmod g-w "$ROOT_PATH/var/drweb" "$ROOT_PATH/var/drweb/ipc"
}
DwUpdateMTA()
{
:
}
DwInstallMtaSENDMAIL()
{
 
    if [ "x$MTA_CONFIG_PATH" != "x" ] && [ -f "$MTA_CONFIG_PATH/sendmail.cf" ] ; then
        SENDMAILCF="$MTA_CONFIG_PATH/sendmail.cf"
    elif [ -f "/usr/local/etc/sendmail.cf" ] ; then
        SENDMAILCF="/usr/local/etc/sendmail.cf"
    elif [ -f "/usr/local/etc/mail/sendmail.cf" ] ; then
        SENDMAILCF="/usr/local/etc/mail/sendmail.cf"
    elif [ -f "/etc/mail/sendmail.cf" ] ; then
        SENDMAILCF="/etc/mail/sendmail.cf"
    elif [ -f "/etc/sendmail.cf" ] ; then
        SENDMAILCF="/etc/sendmail.cf"
    else
        echo ""
        echo "Error: sendmail.cf was not found !"
        echo "Info: you can specify the MTA_CONFIG_PATH environment variable."
        return 1
    fi
    
    /usr/sbin/sendmail -d0.1 -bp >/dev/null 2>&1
    if [ $? -eq 0 ] ; then
        IS_MILTER=`/usr/sbin/sendmail -d0.1 -bp |$GREP MILTER`
        if [ "x$IS_MILTER" != "x"  ] ; then 
            echo "Info: Milter API found"
            
            
            SENDMAIL_VERSION=`/usr/sbin/sendmail -d0.1 -bp|$GREP Version | $SED -e 's/\+.*//g'`
            VERSION_NUM=`echo $SENDMAIL_VERSION|$SED 's/^Version \([0123456789]*\)\.\([0123456789]*\)\.\([0123456789]*\)$/\1/'`
            V2=`echo $SENDMAIL_VERSION|$SED 's/^Version \([0123456789]*\)\.\([0123456789]*\)\.\([0123456789]*\)$/\2/'`
            V3=`echo $SENDMAIL_VERSION|$SED 's/^Version \([0123456789]*\)\.\([0123456789]*\)\.\([0123456789]*\)$/\3/'`
            VERSION_NUM=`expr $VERSION_NUM \* 10000 + $V2 \* 100 + $V3`
#            if [ "$VERSION_NUM" -lt "81301" ]; then
#                echo ""
#                echo "Version Sendmail is less than 8.13.1 -> setting HaveInsHeader to no."
#                echo ""
#                (
#                    DW_ROOT_PATH=$ROOT_PATH;
#                    export DW_ROOT_PATH;
#                    cd "$ROOT_PATH/$SCRIPTS_DIR"
#                    perl -e 'use dwconfig;my ($dw_root_path,$dw_bin_dir,$dw_etc_dir,$dw_conf_file) = DwInit();my $drweb_conf = DwLoadFile("$dw_etc_dir/$dw_conf_file");$drweb_conf = DwModify($drweb_conf,"Milter","HaveInsHeader","no","bool");DwSaveFile("$dw_etc_dir/$dw_conf_file",$drweb_conf);'
#                    )
#            fi
        else
            echo ""
            echo "Your Sendmail MTA compiled without MILTER API."
            echo ""
            return 2
        fi
    fi

    IF_FILTER=`cat $SENDMAILCF |$GREP -v "#" |$GREP "drweb-milter"`
    if [ "x$IF_FILTER" != "x"  ] ; then
        echo ""
        echo "Info: $SENDMAILCF already contains drweb filter, skipping patching $SENDMAILCF!"
        return 10
    fi

    DwBackup $SENDMAILCF || return 1

    echo "#######################################" > $SENDMAILCF.tmp
    echo "### ADDED BY MAILD-SENDMAIL INSTALL ###" >> $SENDMAILCF.tmp
    echo "#######################################" >> $SENDMAILCF.tmp
    echo "O InputMailFilters=drweb-milter" >> $SENDMAILCF.tmp
    echo "O Milter.LogLevel=6" >> $SENDMAILCF.tmp
    echo "Xdrweb-milter,  S=inet:3001@127.0.0.1, F=T, T=C:1m;S:5m;R:5m;E:1h" >> $SENDMAILCF.tmp
    echo "Tdrweb" >> $SENDMAILCF.tmp
    echo "O Milter.macros.connect=_,{if_addr}" >> $SENDMAILCF.tmp
    echo "O Milter.macros.envfrom=i,{auth_type}" >> $SENDMAILCF.tmp
    echo "#######################################" >> $SENDMAILCF.tmp
    cat $SENDMAILCF >> $SENDMAILCF.tmp
    mv -f $SENDMAILCF.tmp $SENDMAILCF
    if [ $? -ne 0 ] ; then                                                              
        echo ""
        echo "Error: can't save settings from $SENDMAILCF.tmp!"
        echo ""                                                                         
        return 5
    fi

    #fix #16842 - wrong permissions after update
    chmod 0644 $SENDMAILCF

    echo "$SENDMAILCF successfull patched"
    MTA_CONFIGURE="yes"
}


############ START ##############

if [ "x$1" = "xupdate" ] ; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will update configuration of SENDMAIL.
    echo
    while true ; do
        echo "Do you wish to continue?"
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DWPrepareMta
    DwUpdateMTA
fi


DwPrepareMta

#configure MTA
if  [ "x$SKIP_MTA_CONFIG" != "xyes" ]; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will configure SENDMAIL.
    echo
    while true ; do
        echo "Perform MTA configuration?"
	echo "Please enter yes or no."
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DwInstallMtaSENDMAIL
fi

if [ "SENDMAIL" != "MAILDESK" ]; then
  if [ "$MTA_CONFIGURE" = "no" ]
  then
    echo "Please, refer to documentation on SENDMAIL adjustment residing in $ROOT_PATH/$DOC_DIR directory."
  else
    echo "Please, restart SENDMAIL manually."
  fi
fi

